# -*- coding: utf-8 -*-
## $Id: test_batching.py 12315 2014-12-24 16:00:58Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Unit tests for product data processors.
"""
import os
import shutil
import tempfile
import unittest
import datetime
import grok
from decimal import Decimal
from time import time
from zope.event import notify
from zope.component import createObject, queryUtility
from zope.component.hooks import setSite, clearSite
from zope.catalog.interfaces import ICatalog
from zope.interface.verify import verifyClass, verifyObject
from zope.securitypolicy.interfaces import IPrincipalRoleManager

from waeup.ikoba.app import Company
from waeup.ikoba.interfaces import IBatchProcessor, FatalCSVError, IUserAccount
from waeup.ikoba.products.batching import ProductProcessor
from waeup.ikoba.products.product import Product
from waeup.ikoba.products.productoptions import ProductOptionEntry
from waeup.ikoba.testing import FunctionalLayer, FunctionalTestCase

PRODUCT_SAMPLE_DATA = open(
    os.path.join(os.path.dirname(__file__), 'sample_product_data.csv'),
    'rb').read()

PRODUCT_HEADER_FIELDS = PRODUCT_SAMPLE_DATA.split(
    '\n')[0].split(',')

PRODUCT_SAMPLE_DATA_UPDATE = open(
    os.path.join(os.path.dirname(__file__), 'sample_product_data_update.csv'),
    'rb').read()

PRODUCT_HEADER_FIELDS_UPDATE = PRODUCT_SAMPLE_DATA_UPDATE.split(
    '\n')[0].split(',')

class ProductImportExportSetup(FunctionalTestCase):

    layer = FunctionalLayer

    def setUp(self):
        super(ProductImportExportSetup, self).setUp()
        self.dc_root = tempfile.mkdtemp()
        self.workdir = tempfile.mkdtemp()
        app = Company()
        app['datacenter'].setStoragePath(self.dc_root)
        self.getRootFolder()['app'] = app
        self.app = self.getRootFolder()['app']
        setSite(app)

        self.logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'main.log')
        return

    def tearDown(self):
        super(ProductImportExportSetup, self).tearDown()
        shutil.rmtree(self.workdir)
        shutil.rmtree(self.dc_root)
        clearSite()
        return

    def setup_for_export(self):
        product = Product()
        product.product_id = u'LIC'
        options = ProductOptionEntry()
        options.title = u'Base Fee'
        options.fee = Decimal('800.6')
        options.currency = 'usd'
        product.options = [options,]
        self.app['products'][product.product_id] = self.product = product
        self.outfile = os.path.join(self.workdir, 'myoutput.csv')
        role_manager = IPrincipalRoleManager(product)
        role_manager.assignRoleToPrincipal(u'johnsrole', u'john')
        return


class ProductProcessorTest(ProductImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(ProductProcessorTest, self).setUp()

        # Add product with subobjects
        product = Product()
        self.app['products'].addProduct(product)
        product.title = u'Our Product'
        notify(grok.ObjectModifiedEvent(product))
        self.product = self.app['products'][product.product_id]

        self.processor = ProductProcessor()
        self.csv_file = os.path.join(self.workdir, 'sample_product_data.csv')
        self.csv_file_update = os.path.join(
            self.workdir, 'sample_product_data_update.csv')
        open(self.csv_file, 'wb').write(PRODUCT_SAMPLE_DATA)
        open(self.csv_file_update, 'wb').write(PRODUCT_SAMPLE_DATA_UPDATE)

    def test_interface(self):
        # Make sure we fulfill the interface contracts.
        assert verifyObject(IBatchProcessor, self.processor) is True
        assert verifyClass(
            IBatchProcessor, ProductProcessor) is True

    def test_parentsExist(self):
        self.assertFalse(self.processor.parentsExist(None, dict()))
        self.assertTrue(self.processor.parentsExist(None, self.app))

    def test_entryExists(self):
        assert self.processor.entryExists(
            dict(product_id='ID_NONE'), self.app) is False
        assert self.processor.entryExists(
            dict(product_id='NA'), self.app) is True

    def test_getParent(self):
        parent = self.processor.getParent(None, self.app)
        assert parent is self.app['products']

    def test_getEntry(self):
        assert self.processor.getEntry(
            dict(product_id='ID_NONE'), self.app) is None
        assert self.processor.getEntry(
            dict(product_id=self.product.product_id), self.app) is self.product

    def test_addEntry(self):
        new_product = Product()
        new_product.product_id = u'NEW'
        self.processor.addEntry(
            new_product, dict(), self.app)
        assert len(self.app['products'].keys()) == 2

    def test_checkConversion(self):
        errs, inv_errs, conv_dict = self.processor.checkConversion(
            dict(product_id='ABC'))
        self.assertEqual(len(errs),0)
        return

    def test_delEntry(self):
        assert self.product.product_id in self.app['products'].keys()
        self.processor.delEntry(
            dict(product_id=self.product.product_id), self.app)
        assert self.product.product_id not in self.app['products'].keys()

    def test_import(self):
        num, num_warns, fin_file, fail_file = self.processor.doImport(
            self.csv_file, PRODUCT_HEADER_FIELDS)
        self.assertEqual(num_warns,1)
        self.assertEqual(len(self.app['products']), 4)
        self.assertEqual(self.app['products']['ABC'].title,'ABC title')
        logcontent = open(self.logfile).read()
        # Logging message from updateEntry
        self.assertTrue(
            'INFO - system - Product Processor - sample_product_data - '
            'EFG - updated: product_id=EFG, title=EFG title, '
            'contract_category=license\n'
            in logcontent)
        failcontent = open(fail_file).read()
        self.assertTrue(
            'ABC,Duplicate product,This object already exists.'
            in failcontent)
        shutil.rmtree(os.path.dirname(fin_file))

    def test_import_update(self):
        num, num_warns, fin_file, fail_file = self.processor.doImport(
            self.csv_file, PRODUCT_HEADER_FIELDS)
        shutil.rmtree(os.path.dirname(fin_file))
        num, num_warns, fin_file, fail_file = self.processor.doImport(
            self.csv_file_update, PRODUCT_HEADER_FIELDS_UPDATE, 'update')
        self.assertEqual(num_warns,1)
        # title has changed
        self.assertEqual(self.app['products']['ABC'].title,'ABC new title')
        logcontent = open(self.logfile).read()
        # Logging message from updateEntry
        self.assertTrue(
            'INFO - system - Product Processor - sample_product_data_update - '
            'ABC - updated: product_id=ABC, title=ABC new title\n'
            in logcontent)
        failcontent = open(fail_file).read()
        self.assertTrue(
            'XYZ,Non-existing product,Cannot update: no such entry'
            in failcontent)
        shutil.rmtree(os.path.dirname(fin_file))
